<?php
/**
 * @version 2.0 
 * @author Dovalovszki Tamás 
 * @package Description library
 * @license All Rights Reserved - Minden jog fenntartva
 * @copyright (C) 2018 Dovalovszki Tamás - All rights reserved.
 **/

/*
    Meghívása:
    if (!class_exists ('JDescription', false)) {
        include_once(dirname(__FILE__) .DIRECTORY_SEPARATOR. 'description.php');
    }
    $desc = new JDescription('<path or url>');
    $title = $desc->getTitle('<filename>');
*/


class JDescription extends stdClass
{
    private $_descFile = '';
    public  $name = '';
    public  $title = '';
    private $_fileName = 'descript.ion';
    private $_items = array();
    private $_encoding = null;
    private $is_local = true;
    private $silent = false;
    

	/**
	 * JDescription::__construct()
	 * 
	 * @param mixed $file Description file (include full path)
	 * @param bool $silent Set Silent mode - default: false  
	 * @return
	 */
	public function __construct($file = null, $silent = false)
	{
        if ($silent) {
            $this->silent = true;
        }
        
        if ($file != null) {
            if (substr($file, -1) == DIRECTORY_SEPARATOR) {
                $file = substr($file, 0, -1);
            }
            if (substr($file, 0, 4) == 'http') {
                $this->is_local = false;
            }
            
            $this->_descFile = $file .DIRECTORY_SEPARATOR. $this->_fileName;
            
            if ($this->is_local) {
                $tmp = explode(DIRECTORY_SEPARATOR, $file);
                $this->name = $tmp[count($tmp)-1];
                
                unset($tmp[count($tmp)-1]);
                $tmp = implode(DIRECTORY_SEPARATOR, $tmp);
                $tmp = $this->getLocalItems($tmp .DIRECTORY_SEPARATOR. $this->_fileName);
                if (isset($tmp[$this->name])) {
                    $this->title = $tmp[$this->name];
                }
                
                $this->_items = $this->getLocalItems();
            } else {
                $tmp = explode(DIRECTORY_SEPARATOR, $file);
                $this->name = $tmp[count($tmp)-1];
                $pos = strpos($this->name, '.');
                if ($pos !== false) {
                    $this->name = '';
                } else {
                    unset($tmp[count($tmp)-1]);
                    $tmp = implode(DIRECTORY_SEPARATOR, $tmp);
                    $tmp = $this->getLocalItems($tmp .DIRECTORY_SEPARATOR. $this->_fileName);
                    if (isset($tmp[$this->name])) {
                        $this->title = $tmp[$this->name];
                    }
                }
                $this->_items = $this->getLocalItems();
            }
        } else {
            if (!$this->silent) {
                echo "\r\n" . '<!-- JDescripion: Hibás hívás! Nincs megadva a descrition fájl helye. -->' . "\r\n";
            }
        }
    }

    /**
     * JDescription::importLines()
     * 
     * @param mixed $lines
     * @return
     */
    private function importLines($lines)
    {
        $items = array();
        for ($i = 0; $i <= count($lines)-1; $i++) {
            $text = trim($lines[$i]);
            $code = array('UTF-8', 'ISO-8859-2');
            $this->_encoding = mb_detect_encoding($text, $code);
            if ($this->_encoding != 'UTF-8') {
                $text = iconv($this->_encoding, 'UTF-8', $text);
            }
            if (substr($text, 0, 1) == '"') {
                $text = substr($text, 1);
                $pos = strpos($text, '"');
                $id = substr($text, 0, $pos);
                $title = substr($text, $pos + 2);
            } else {
                $pos = strpos($text, ' ');
                $id = substr($text, 0, $pos);
                $title = substr($text, $pos + 1);
            }
            $title = str_replace('\n', '<br />', $title);
            $title = str_replace('Â', '', $title);
            $items[$id] = $title;
        }
        return $items;
    }

    /**
     * JDescription::getLocalItems()
     * 
     * @param mixed $path
     * @return
     */
    private function getLocalItems($path = null)
    {
        $items = array();
        if ($path == null) {
            $path = $this->_descFile;
        }
        $lines = @file( $path );     
        if ($lines) {
            $items = $this->importLines($lines);
        } else {
            $this->_encoding = 'UTF-8';
            if ($path == $this->_descFile) {
                if (!$this->silent) {
                    echo "\r\n" . '<!-- JDescripion: A ' . $path . ' fájl nem található. -->' . "\r\n";
                }
            }
        }
        return $items;
    }

    /**
     * JDescription::getRemoteItems()
     * 
     * @return
     */
    private function getRemoteItems()
    {
        $sets = stream_context_create(
            array('http'=> array('timeout' => 5)
        ));
        $lines = @file_get_contents($this->_descFile, false, $sets);
        if ($lines != null) {
            $this->importLines($lines);
        } else {
            $this->_encoding = 'UTF-8';
            if (!$this->silent) {
                echo "\r\n" . '<!-- JDescripion: A távoli, ' . $this->_descFile . ' fájl nem található, vagy nem olvasható. -->' . "\r\n";
                echo '<!-- Győződjön meg róla, hogy a könyvtárban nincs index.htm, index.html, vagy index.php fájl, valamint hogy a .htaccess fájlban engedélyezve van a könyvtár listázása. (Options +Indexes)  -->' . "\r\n";
            }
        }
    }

    /**
     * JDescription::getName()
     * 
     * @param mixed $id
     * @return
     */
    public function getName($id = null)
    {
        $id = explode(DIRECTORY_SEPARATOR, $id);
        $id = $id[ count($id)-1 ];
        return $id;
    }

    /**
     * JDescription::getTitle()
     * 
     * @param mixed $id
     * @return
     */
    public function getTitle($id = null)
    {
        if (isset($this->_items[$id]) ) 
        {
            return $this->_items[$id];
        } else {
            return $id;
        }
    }
    
    /**
     * JDescription::update()
     * 
     * @param mixed $id
     * @param mixed $title
     * @return
     */
    public function update($id, $title)
    {
        $this->_items[$id] = $title;
    }

    /**
     * JDescription::updateKey()
     * 
     * @param mixed $old
     * @param mixed $new
     * @return
     */
    public function updateKey($old, $new)
    {
        $tmp = $this->_items[$old];
        unset($this->_items[$old]);
        $this->_items[$new] = $tmp;
    }
    
    /**
     * JDescription::store()
     * 
     * @return
     */
    public function store()
    {
		if ($this->is_local) {
            $file = fopen($this->_descFile, 'w+');
            while($element = current($this->_items)) {
                $id = key($this->_items);
                $title = $this->_items[$id];
                if ($this->_encoding != 'UTF-8') {
                    $title = iconv('UTF-8', 'ISO8859-2', $title);
                }
                $pos = strpos($id, ' ');
                if ($pos !== false) {
                    $id = '"'. $id . '"';
                }
                $path = str_replace($this->_fileName, '', $this->_descFile);
                fwrite($file, $id . ' ' . $title . "\r\n");    
                next($this->_items);
            }
            fclose($file);
            chmod($this->_descFile, 0775);
		}
    }
    
    /**
     * JDescription::searchTitle()
     * 
     * @param string $text
     * @return
     */
    function searchTitle($text = '')
    {
        if ($text != '') {
            $results = array();
            $text = mb_strtoupper($text);
            foreach ($this->_items as $key => $value) {
                $pos = strpos(mb_strtoupper($value), $text);
                if ($pos !== false) {
                    $results[] = array('file' => $key, 'title' => $value);
                }
            }
            return $results;
        } else {
            return array();
        }
    }
}
?>